import os
import typing

import markovify
import nltk
import regex

from .odt import Parser


def path_to_str(path: str) -> str:
    path_str: str = os.fspath(path)
    if path_str.endswith('odt'):
        return Parser().extract(path)
    else:
        with open(path, "r", encoding='utf-8-sig') as f:
            return f.read()


alpha_re = regex.compile(r'^[\w,–]+$')


def generate_from_str(text: str):
    text = text.lower()
    words = [
        [word for word in nltk.word_tokenize(sentence, 'polish') if alpha_re.match(word)]
        for sentence in nltk.sent_tokenize(text, 'polish')
    ]
    return markovify_generate(words)


def generate_from_file(path: str) -> typing.List:
    text = path_to_str(path)

    return generate_from_str(text)


def markovify_generate(tokens, max_results=5, max_attempts=100) -> typing.List:
    m = markovify.Text(None, state_size=1, parsed_sentences=tokens)
    result = []
    attempts = 0

    while len(result) < max_results and attempts < max_attempts:
        attempts += 1
        sentence = m.make_sentence(max_overlap_total=3)
        if sentence is not None:
            result.append(sentence)

    return result



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['mattes_dada']

package_data = \
{'': ['*']}

install_requires = \
['markovify>=0.7.1,<0.8.0', 'nltk>=3.4,<4.0', 'regex>=2019.2,<2020.0']

entry_points = \
{'console_scripts': ['mdada = mattes_dada.cmd:main']}

setup_kwargs = {
    'name': 'mattes-dada',
    'version': '0.1.0',
    'description': 'Semi-random text generator using Markov chains',
    'long_description': "Matte's Dada\n===\n\nSemi-random text generator using Markov chains\n\nRun\n==\n\n```shell\nmdada input_file\n```\n\nwhere input_file is a path to either a txt ot odt file\n",
    'author': 'Raphael Krupinski',
    'author_email': None,
    'url': 'https://github.com/mattesilver/dada',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from sys import argv

from .dada import generate_from_file
from .nltk_data import init_nltk_data


def main():
    init_nltk_data()

    if len(argv) < 2:
        print("pass file name")
        exit(-1)

    for sent in generate_from_file(argv[1]):
        print(sent)


if __name__ == '__main__':
    main()



import datetime
import os.path
import platform

import nltk.data

_data = {
    'punkt': 'tokenizers/punkt'
}

_locations = {
    'Darwin': '$HOME/Library/Mattes-dada',
    'Linux': '$HOME/.mattes-dada',
    'Windows': '%APPDATA%/Mattes-dada',
}

_location: str = None


def _init_location() -> None:
    """Select data directory based on operating system and add it to nltk search path."""
    system = platform.system()
    if system not in _locations:
        raise Exception('Unknown operating system:', system)
    else:
        global _location
        _location = os.path.expandvars(_locations[system])
    nltk.data.path.insert(0, _location)


def _is_file_expired(full_path, expiry: datetime.datetime) -> bool:
    """Check if given file was modified before 'expiry'

    :return: True if the file was modified before the expiry time, False otherwise
    """

    mod_time = os.stat(full_path).st_mtime
    return mod_time < expiry.timestamp()


def download(id_str: str) -> None:
    """Use nltk.download function to download data package identified by id_str.

    :param id_str: nltk data package name as per nltk_data index.xml
    """
    nltk.download(id_str, download_dir=_location, quiet=True, halt_on_error=False, raise_on_error=True)


def _update(id_str: str, path: str, expiry: datetime.datetime) -> None:
    """Check if the package has been downloaded and if its modification time is not before expiry time. If it's not, try downloading it

    :param id_str: package id
    :param path: relative package path, where nltk downloader unzips the package
    :param expiry: expiry time
     """
    full_path = os.path.join(_location, path)
    if not os.path.exists(full_path) or _is_file_expired(full_path, expiry):
        download(id_str)


def expiry_date(max_age_days: int) -> datetime.datetime:
    """Calculate expiry time, by subtracting max_age_days days from the current time.

    :param max_age_days: number of days
    """
    delta = datetime.timedelta(days=max_age_days)
    return datetime.datetime.utcnow() - delta


def init_nltk_data(max_age_days: int = 30) -> None:
    """Download nltk data packages or try updating them if they are old.

    :param max_age_days: if the file is older than this many days, it is considered old
    """
    _init_location()
    expiry = expiry_date(max_age_days)
    for id, path in _data.items():
        _update(id, path, expiry)
